from collections import defaultdict
from pathlib import Path
from typing import DefaultDict, Dict, FrozenSet, Iterable, Mapping, Optional, Set, Tuple

from rich.console import Console
from rich.padding import Padding
from rich.progress import Progress, track
from rich.table import Table
from typer import Argument, Option
from yaml import safe_load as yaml_safe_load

from deckz.cli import app
from deckz.paths import GlobalPaths, Paths
from deckz.targets import Targets


@app.command()
def deps(
    section: Optional[str] = Argument(None, help="Section to study"),
    flavor: Optional[str] = Argument(None, help="Flavor to study"),
    unused: bool = Option(True, help="Display the unused flavors"),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Display information about shared sections and flavors usage."""
    paths = GlobalPaths.from_defaults(workdir)
    with Progress() as progress:
        targets_progress = progress.add_task("Retrieving targets files", start=False)
        sections_progress = progress.add_task("Retrieving section files", start=False)

        targets_paths = list(paths.git_dir.glob("**/targets.yml"))
        progress.update(
            targets_progress, total=len(targets_paths), completed=len(targets_paths)
        )
        progress.start_task(targets_progress)
        ymls = paths.shared_latex_dir.glob("**/*.yml")
        section_paths = []
        for yml in ymls:
            with yml.open(encoding="utf8") as fh:
                content = yaml_safe_load(fh)
                if not isinstance(content, dict):
                    continue
                if {"title", "flavors"}.issubset(content):
                    section_paths.append(yml)

        progress.update(
            sections_progress, total=len(section_paths), completed=len(section_paths)
        )
        progress.start_task(sections_progress)

    all_targets_dependencies: Dict[str, Dict[str, Set[str]]] = defaultdict(dict)
    by_sections: DefaultDict[str, DefaultDict[str, Set[Tuple[str, str]]]] = defaultdict(
        lambda: defaultdict(set)
    )
    for targets_path in track(targets_paths, description="Processing targets files"):
        _process_targets(targets_path, all_targets_dependencies, by_sections)

    if unused:
        _print_unused_report(section_paths, by_sections, paths)
    if section is not None:
        _print_section_report(section, flavor, by_sections)


def _compute_shared_dependencies(
    dependencies: Set[Path], paths: GlobalPaths
) -> Set[str]:
    shared_dependencies = set()
    for dependency in dependencies:
        try:
            relative_dependency = dependency.relative_to(paths.shared_latex_dir)
            shared_dependencies.add(str(relative_dependency))
        except ValueError:
            pass
    return shared_dependencies


def _process_targets(
    targets_path: Path,
    all_targets_dependencies: Dict[str, Dict[str, Set[str]]],
    by_sections: Mapping[str, Mapping[str, Set[Tuple[str, str]]]],
) -> None:
    paths = Paths.from_defaults(targets_path.parent)
    targets_name = str(paths.current_dir.relative_to(paths.git_dir))
    targets = Targets.from_file(paths)
    for target in targets:
        for section_path, dependencies in target.section_dependencies.items():
            section_flavors = target.section_flavors[section_path]
            shared = False
            for dependency_path in dependencies.used:
                if _relative_to_shared_latex(paths, str(dependency_path)) is not None:
                    shared = True
                    break
            if shared:
                for section_flavor in section_flavors:
                    by_sections[section_path][section_flavor].add(
                        (targets_name, target.name)
                    )
        shared_dependencies = _compute_shared_dependencies(
            target.dependencies.used, paths
        )
        all_targets_dependencies[targets_name][target.name] = shared_dependencies


def _print_changed_report(
    touched: FrozenSet[str], all_targets_dependencies: Dict[str, Dict[str, Set[str]]]
) -> None:
    console = Console()
    console.print()
    console.print()
    console.rule("[bold]Decks affected by uncommited changes", align="left")
    console.print()
    table = Table("Deck", "Targets")
    for targets_name, targets_dependencies in all_targets_dependencies.items():
        touched_targets = []
        for target_name, target_dependencies in targets_dependencies.items():
            if touched.intersection(target_dependencies):
                touched_targets.append(target_name)
        if touched_targets:
            table.add_row(targets_name, " ".join(touched_targets))
    console.print(Padding(table if table.row_count else "None.", (0, 0, 0, 2)))


def _print_unused_report(
    section_paths: Iterable[Path],
    by_sections: Mapping[str, Mapping[str, Set[Tuple[str, str]]]],
    global_paths: GlobalPaths,
) -> None:
    console = Console()
    console.print()
    console.print()
    console.rule("[bold]Unused flavors", align="left")
    console.print()
    table = Table("Section", "Flavors")
    flavors = defaultdict(set)
    for section_path in section_paths:
        with section_path.open(encoding="utf8") as fh:
            section_config = yaml_safe_load(fh)
        section_name = "/".join(
            section_path.relative_to(global_paths.shared_latex_dir).parts[:-1]
        )
        flavors[section_name].update(section_config["flavors"])
    for section_name, section_flavors in sorted(flavors.items()):
        unused_flavors = [
            flavor
            for flavor in section_flavors
            if not by_sections[section_name][flavor]
        ]
        if unused_flavors:
            table.add_row(section_name, " ".join(sorted(unused_flavors)))
    console.print(Padding(table if table.row_count else "None.", (0, 0, 0, 2)))


def _print_section_report(
    section: str,
    flavor: Optional[str],
    by_sections: Mapping[str, Mapping[str, Set[Tuple[str, str]]]],
) -> None:
    console = Console()
    console.print()
    console.print()
    title = section
    if flavor is not None:
        title += f" {flavor}"
    console.rule(f"[bold]Decks depending on [italic]{title}", align="left")
    console.print()
    table = Table("Deck", "Targets")
    if flavor is None:
        flavors_dependencies = by_sections[section]
        for deck_path, target_name in sorted(set.union(*flavors_dependencies.values())):
            table.add_row(deck_path, target_name)
    else:
        flavors_dependencies = by_sections[section]
        for deck_path, target_name in sorted(flavors_dependencies[flavor]):
            table.add_row(deck_path, target_name)
    console.print(Padding(table if table.row_count else "None.", (0, 0, 0, 2)))


def _relative_to_shared_latex(paths: GlobalPaths, file_path: str) -> Optional[str]:
    try:
        return str((paths.git_dir / file_path).relative_to(paths.shared_latex_dir))
    except ValueError:
        return None



from logging import getLogger
from pathlib import Path
from typing import List, Optional

from rich.progress import BarColumn, Progress

from deckz.building import Builder
from deckz.config import get_config
from deckz.paths import GlobalPaths, Paths
from deckz.settings import Settings
from deckz.standalones import StandalonesBuilder
from deckz.targets import Targets

_logger = getLogger(__name__)


def run(
    paths: Paths,
    build_handout: bool,
    build_presentation: bool,
    build_print: bool,
    target_whitelist: Optional[List[str]] = None,
) -> None:
    config = get_config(paths)
    targets = Targets.from_file(paths=paths, whitelist=target_whitelist)
    settings = Settings.from_global_paths(paths)
    StandalonesBuilder(settings, paths).build()
    Builder(
        config,
        settings,
        paths,
        targets,
        build_handout=build_handout,
        build_presentation=build_presentation,
        build_print=build_print,
    ).build()


def run_file(
    latex: str,
    paths: Paths,
    build_handout: bool,
    build_presentation: bool,
    build_print: bool,
) -> None:
    config = get_config(paths)
    targets = Targets.from_data(
        [dict(name="section", title="section", sections=[dict(path=latex)])],
        paths=paths,
    )
    settings = Settings.from_global_paths(paths)
    StandalonesBuilder(settings, paths).build()
    Builder(
        config,
        settings,
        paths,
        targets,
        build_handout=build_handout,
        build_presentation=build_presentation,
        build_print=build_print,
    ).build()


def run_all(
    directory: Path,
    build_handout: bool,
    build_presentation: bool,
    build_print: bool,
) -> None:

    paths = GlobalPaths.from_defaults(directory)
    settings = Settings.from_global_paths(paths)
    StandalonesBuilder(settings, paths).build()
    targets_paths = list(paths.git_dir.glob("**/targets.yml"))
    with Progress(
        "[progress.description]{task.description}",
        BarColumn(),
        "[progress.percentage]{task.percentage:>3.0f}%",
    ) as progress:
        task_id = progress.add_task("Building decks…", total=len(targets_paths))
        for target_paths in targets_paths:
            deck_paths = Paths.from_defaults(target_paths.parent)
            config = get_config(deck_paths)
            targets = Targets.from_file(paths=deck_paths)
            builder = Builder(
                config,
                settings,
                deck_paths,
                targets,
                build_handout=build_handout,
                build_presentation=build_presentation,
                build_print=build_print,
            )
            result = builder.build()
            if not result:
                break
            progress.update(task_id, advance=1)


def run_standalones(directory: Path) -> None:
    paths = GlobalPaths.from_defaults(directory)
    settings = Settings.from_global_paths(paths)
    standalones_builder = StandalonesBuilder(settings, paths)
    standalones_builder.build()



from filecmp import cmp
from functools import cached_property
from os import unlink
from os.path import join as path_join
from pathlib import Path
from shutil import move
from tempfile import NamedTemporaryFile
from typing import Any, Callable, List, Tuple

from jinja2 import BaseLoader, Environment, TemplateNotFound
from yaml import safe_load

from deckz.paths import Paths
from deckz.settings import Settings


class AbsoluteLoader(BaseLoader):
    def get_source(
        self, environment: Environment, template: str
    ) -> Tuple[str, str, Callable[[], bool]]:
        template_path = Path(template)
        if not template_path.exists():
            raise TemplateNotFound(template)
        mtime = template_path.stat().st_mtime
        source = template_path.read_text(encoding="utf8")
        return (
            source,
            str(template_path),
            lambda: mtime == template_path.stat().st_mtime,
        )


class Renderer:
    def __init__(self, paths: Paths, settings: Settings):
        self._paths = paths
        self._settings = settings

    def render(
        self, *, template_path: Path, output_path: Path, **template_kwargs: Any
    ) -> None:
        template = self._env.get_template(str(template_path))
        try:
            with NamedTemporaryFile("w", encoding="utf8", delete=False) as fh:
                fh.write(template.render(**template_kwargs))
                fh.write("\n")
            if not output_path.exists() or not cmp(fh.name, str(output_path)):
                move(fh.name, output_path)
        finally:
            try:
                unlink(fh.name)
            except FileNotFoundError:
                pass

    @cached_property
    def _env(self) -> Environment:
        env = Environment(
            loader=AbsoluteLoader(),
            block_start_string=r"\BLOCK{",
            block_end_string="}",
            variable_start_string=r"\V{",
            variable_end_string="}",
            comment_start_string=r"\#{",
            comment_end_string="}",
            line_statement_prefix="%%",
            line_comment_prefix="%#",
            trim_blocks=True,
            autoescape=False,
        )
        env.filters["camelcase"] = self._to_camel_case
        env.filters["path_join"] = lambda paths: path_join(*paths)
        env.filters["image"] = self._img
        return env

    def _to_camel_case(self, string: str) -> str:
        return "".join(substring.capitalize() or "_" for substring in string.split("_"))

    def _img(self, args: List[Any]) -> str:
        if not isinstance(args, list):
            args = [args]
        path = args[0]
        modifier = args[1] if len(args) > 1 else ""
        scale = "{%.2f}" % args[2] if len(args) > 2 else "{1}"
        lang = args[3] if len(args) > 3 else "fr"

        metadata_path = (self._paths.shared_dir / path).with_suffix(".yml")
        if metadata_path.exists():
            metadata = safe_load(metadata_path.read_text(encoding="utf8"))

            def get_en_or_fr(key: str) -> str:
                if lang != "fr":
                    key_en = f"{key}_en"
                    return metadata[key_en] if key_en in metadata else metadata[key]
                else:
                    return metadata[key]

            title = self._settings.default_img_values.title.get_default(
                get_en_or_fr("title"), lang
            )
            author = self._settings.default_img_values.author.get_default(
                get_en_or_fr("author"), lang
            )
            license = self._settings.default_img_values.license.get_default(
                get_en_or_fr("license"), lang
            )
            info = f"[{title}, {author}, {license}.]"
        else:
            info = ""

        return f"\\img{modifier}{info}{{{path}}}{scale}"



from logging import getLogger
from typing import Any, Dict, Optional

from requests import post


class GitHubAPI:
    def __init__(self, api_key: str) -> None:
        self._api_key = api_key
        self._logger = getLogger(__name__)

    def _run_query(self, query: str) -> Dict[str, Any]:
        response = post(
            "https://api.github.com/graphql",
            json=dict(query=query),
            headers=dict(Authorization=f"token {self._api_key}"),
        )
        response.raise_for_status()
        return response.json()

    def create_issue(
        self,
        owner: str,
        repo: str,
        title: str,
        body: Optional[str],
        project_number: Optional[int],
    ) -> str:
        self._logger.info("Retrieving repository id")
        repo_id = self.get_repo_id(owner=owner, repo=repo)
        if project_number is not None:
            self._logger.info("Retrieving project id")
            project_id = self.get_project_id(owner=owner, project_number=project_number)
        self._logger.info("Creating issue")
        extras = ""
        if body is not None:
            extras += f'body: "{body}"\n'
        if project_number is not None:
            extras += f'projectIds: ["{project_id}"]\n'
        response = self._run_query(
            f"""
            mutation {{
              createIssue(input: {{
                repositoryId: "{repo_id}"
                title: "{title}"
                {extras}
              }})
              {{
                issue {{
                  url
                }}
              }}
            }}
            """
        )
        return response["data"]["createIssue"]["issue"]["url"]

    def get_repo_id(self, owner: str, repo: str) -> str:
        response = self._run_query(
            f"""
            {{
              repository(name: "{repo}", owner: "{owner}") {{
                id
              }}
            }}
            """
        )
        return response["data"]["repository"]["id"]

    def get_project_id(self, owner: str, project_number: int) -> str:
        response = self._run_query(
            f"""
            {{
              repositoryOwner(login: "{owner}") {{
                ... on User {{
                  project(number: {project_number}) {{
                    id
                  }}
                }}
                ... on Organization {{
                  project(number: {project_number}) {{
                    id
                  }}
                }}
              }}
            }}
            """
        )
        return response["data"]["repositoryOwner"]["project"]["id"]



from logging import getLogger
from pathlib import Path

from typer import Option

from deckz.cli import app
from deckz.paths import GlobalPaths
from deckz.watching import watch_standalones as watching_watch_standalones

_logger = getLogger(__name__)


@app.command()
def watch_standalones(
    minimum_delay: int = Option(5, help="Minimum number of seconds before recompiling"),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Compile standalones on change."""
    watching_watch_standalones(
        minimum_delay, workdir, GlobalPaths.from_defaults(workdir)
    )



from enum import Enum
from logging import getLogger
from multiprocessing import Pool, cpu_count
from pathlib import Path
from shutil import copyfile
from typing import Any, Dict, List, Optional

from attr import attrs

from deckz.compiling import CompileResult
from deckz.compiling import compile as compiling_compile
from deckz.exceptions import DeckzException
from deckz.paths import Paths
from deckz.rendering import Renderer
from deckz.settings import Settings
from deckz.targets import Target, Targets
from deckz.utils import copy_file_if_newer


class CompileType(Enum):
    Handout = "handout"
    Presentation = "presentation"
    PrintHandout = "print-handout"


@attrs(auto_attribs=True, frozen=True)
class CompileItem:
    target: Target
    compile_type: CompileType
    toc: bool


class Builder:
    def __init__(
        self,
        latex_config: Dict[str, Any],
        settings: Settings,
        paths: Paths,
        targets: Targets,
        build_presentation: bool,
        build_handout: bool,
        build_print: bool,
    ):
        self._latex_config = latex_config
        self._settings = settings
        self._paths = paths
        self._targets = targets
        self._presentation = build_presentation
        self._handout = build_handout
        self._print = build_print
        self._logger = getLogger(__name__)
        self._renderer = Renderer(paths, settings)

    def _list_items(self) -> List[CompileItem]:
        to_compile = []
        all_target = Target.from_targets(self._targets, "all")
        if self._handout:
            to_compile.append(CompileItem(all_target, CompileType.Handout, True))
        if self._print:
            to_compile.append(CompileItem(all_target, CompileType.PrintHandout, True))
        for target in self._targets:
            if self._presentation:
                to_compile.append(CompileItem(target, CompileType.Presentation, False))
            if self._handout:
                to_compile.append(CompileItem(target, CompileType.Handout, False))
        to_compile.sort(key=lambda item: (item.target.name, item.compile_type.value))
        return to_compile

    def build(self) -> bool:
        items = self._list_items()
        n_outputs = (
            int(self._handout) * (len(self._targets) + 1)
            + int(self._presentation) * len(self._targets)
            + int(self._print)
        )
        self._logger.info(f"Building {len(items)} PDFs used in {n_outputs} outputs")
        with Pool(min(cpu_count(), len(items))) as pool:
            results = pool.map(self._build_item, items)
        for item, result in zip(items, results):
            if item.target is not None:
                compilation = f"{item.target.name}/{item.compile_type.value}"
            else:
                compilation = item.compile_type.value
            if not result.ok:
                self._logger.warning("Compilation %s errored", compilation)
                self._logger.warning(
                    "Captured %s stderr\n%s", compilation, result.stderr
                )
                self._logger.warning(
                    "Captured %s stdout\n%s", compilation, result.stdout
                )
        return all(result.ok for result in results)

    def _get_filename(self, target: Optional[Target], compile_type: CompileType) -> str:
        name = self._latex_config["deck_acronym"]
        if target is not None:
            name += f"-{target.name}"
        name += f"-{compile_type.value}"
        return name.lower()

    def _build_item(self, item: CompileItem) -> CompileResult:
        build_dir = self._setup_build_dir(item.target, item.compile_type)
        filename = self._get_filename(item.target, item.compile_type)
        latex_path = build_dir / f"{filename}.tex"
        build_pdf_path = latex_path.with_suffix(".pdf")
        output_pdf_path = self._paths.pdf_dir / f"{filename}.pdf"
        self._render_latex(item, latex_path)
        copied = self._copy_dependencies(item.target, build_dir)
        self._render_dependencies(copied, build_dir)
        result = compiling_compile(latex_path, self._settings)
        if result.ok:
            self._paths.pdf_dir.mkdir(parents=True, exist_ok=True)
            copyfile(build_pdf_path, output_pdf_path)
        return result

    def _setup_build_dir(
        self, target: Optional[Target], compile_type: CompileType
    ) -> Path:
        target_build_dir = self._paths.build_dir
        if target is not None:
            target_build_dir /= target.name
        target_build_dir /= compile_type.value
        target_build_dir.mkdir(parents=True, exist_ok=True)
        for item in [
            self._paths.shared_img_dir,
            self._paths.shared_tikz_pdf_dir,
            self._paths.shared_code_dir,
        ]:
            self._setup_link(target_build_dir / item.name, item)
        return target_build_dir

    def _render_latex(self, item: CompileItem, output_path: Path) -> None:
        self._renderer.render(
            template_path=self._paths.jinja2_main_template,
            output_path=output_path,
            config=self._latex_config,
            target=item.target,
            handout=item.compile_type
            in [CompileType.Handout, CompileType.PrintHandout],
            toc=item.toc,
            print=item.compile_type is CompileType.PrintHandout,
        )

    def _copy_dependencies(self, target: Target, target_build_dir: Path) -> List[Path]:
        copied = []
        for dependency in target.dependencies.used:
            try:
                link_dir = (
                    target_build_dir
                    / dependency.relative_to(self._paths.shared_dir).parent
                )
            except ValueError:
                link_dir = (
                    target_build_dir
                    / dependency.relative_to(self._paths.current_dir).parent
                )
            link_dir.mkdir(parents=True, exist_ok=True)
            destination = (link_dir / dependency.name).with_suffix(".tex.j2")
            if (
                not destination.exists()
                or destination.stat().st_mtime < dependency.stat().st_mtime
            ):
                copy_file_if_newer(dependency, destination)
                copied.append(destination)
        return copied

    def _render_dependencies(
        self, to_render: List[Path], target_build_dir: Path
    ) -> None:
        for item in to_render:
            self._renderer.render(template_path=item, output_path=item.with_suffix(""))

    def _setup_link(self, source: Path, target: Path) -> None:
        if not target.exists():
            raise DeckzException(
                f"{target} could not be found. Please make sure it exists before "
                "proceeding."
            )
        target = target.resolve()
        if source.is_symlink():
            if source.resolve().samefile(target):
                return
            raise DeckzException(
                f"{source} already exists in the build directory and does not point to "
                f"{target}. Please clean the build directory."
            )
        elif source.exists():
            raise DeckzException(
                f"{source} already exists in the build directory. Please clean the "
                "build directory."
            )
        source.parent.mkdir(parents=True, exist_ok=True)
        source.symlink_to(target)



from pathlib import Path
from shutil import copyfile

from pygit2 import Repository, discover_repository

from deckz.exceptions import DeckzException


def get_git_dir(path: Path) -> Path:
    repository = discover_repository(str(path))
    if repository is None:
        raise DeckzException(
            "Could not find the path of the current git working directory. "
            "Are you in one?"
        )
    return Path(Repository(repository).workdir).resolve()


def copy_file_if_newer(original: Path, copy: Path) -> None:
    if copy.exists() and copy.stat().st_mtime > original.stat().st_mtime:
        return
    else:
        copy.parent.mkdir(parents=True, exist_ok=True)
        copyfile(original, copy)



from pathlib import Path
from typing import List, Optional

from typer import Argument, Option

from deckz.cli import app
from deckz.paths import Paths
from deckz.running import run as running_run


@app.command()
def run(
    targets: Optional[List[str]] = Argument(None, help="Targets to compile"),
    handout: bool = Option(True, help="Produce PDFs without animations"),
    presentation: bool = Option(True, help="Produce PDFs with animations"),
    print: bool = Option(True, help="Produce a printable PDF"),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Compile main targets."""
    paths = Paths.from_defaults(workdir)
    running_run(
        paths=paths,
        build_handout=handout,
        build_presentation=presentation,
        build_print=print,
        target_whitelist=targets,
    )



from logging import getLogger
from pathlib import Path
from typing import List, Optional

from typer import Argument, Option

from deckz.cli import app
from deckz.paths import Paths
from deckz.watching import watch as watching_watch

_logger = getLogger(__name__)


@app.command()
def watch(
    targets: Optional[List[str]] = Argument(None, help="Targets to watch"),
    handout: bool = Option(False, help="Produce PDFs without animations"),
    presentation: bool = Option(True, help="Produce PDFs with animations"),
    print: bool = Option(False, help="Produce a printable PDF"),
    minimum_delay: int = Option(5, help="Minimum number of seconds before recompiling"),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Compile on change."""
    _logger.info("Watching current and shared directories")
    watching_watch(
        minimum_delay=minimum_delay,
        paths=Paths.from_defaults(workdir),
        build_handout=handout,
        build_presentation=presentation,
        build_print=print,
        target_whitelist=targets,
    )



from collections import defaultdict
from logging import getLogger
from pathlib import Path, PurePosixPath
from typing import (
    Any,
    DefaultDict,
    Dict,
    FrozenSet,
    Iterable,
    Iterator,
    List,
    Optional,
    Set,
    Tuple,
    Union,
)

from attr import Attribute, Factory, attrib, attrs
from pydantic import BaseModel, ValidationError
from yaml import safe_load as safe_load

from deckz.exceptions import DeckzException
from deckz.paths import Paths

_logger = getLogger(__name__)


@attrs(auto_attribs=True)
class Title:
    title: str
    level: int


class DirSectionConfig(BaseModel):
    default_titles: Optional[Dict[str, str]]
    flavors: Dict[str, List[Union[str, Dict[str, Optional[str]]]]]
    title: str

    @classmethod
    def from_yaml_file(cls, path: Path) -> "DirSectionConfig":
        try:
            return cls.parse_obj(safe_load(path.read_text(encoding="utf8")))
        except (IOError, ValidationError) as e:
            raise DeckzException(f"Could not load {path} section config") from e


Content = str
ContentOrTitle = Union[Content, Title]


@attrs(auto_attribs=True)
class Part:
    title: Optional[str]
    sections: List[ContentOrTitle] = Factory(list)


@attrs(auto_attribs=True)
class Dependencies:
    used: Set[Path] = Factory(set)
    missing: Set[str] = Factory(set)
    unused: Set[Path] = Factory(set)

    def update(self, other: "Dependencies") -> None:
        self.used |= other.used
        self.missing |= other.missing
        self.unused |= other.unused - self.used
        self.unused -= other.used

    @staticmethod
    def merge(*dependencies_list: "Dependencies") -> "Dependencies":
        dependencies = Dependencies()
        for ds in dependencies_list:
            dependencies.used |= ds.used
            dependencies.missing |= ds.missing
            dependencies.unused |= ds.unused - dependencies.used
            dependencies.unused -= ds.used
        return dependencies

    def merge_dicts(
        *dependencies_dicts: Dict[str, "Dependencies"]
    ) -> Dict[str, "Dependencies"]:
        keys = set.union(*(set(d) for d in dependencies_dicts))
        merged_dict = {}
        for key in keys:
            merged_dict[key] = Dependencies.merge(
                *(d[key] for d in dependencies_dicts if key in d)
            )
        return merged_dict


@attrs(auto_attribs=True)
class Target:
    name: str
    dependencies: Dependencies
    parts: List[Part]
    section_dependencies: DefaultDict[str, Dependencies]
    section_flavors: DefaultDict[str, Set[str]]

    @classmethod
    def from_targets(cls, targets: Iterable["Target"], name: str) -> "Target":
        dependencies = Dependencies.merge(*(t.dependencies for t in targets))
        parts = [p for t in targets for p in t.parts]
        section_dependencies: DefaultDict[str, Dependencies] = defaultdict(Dependencies)
        for t in targets:
            for k, v in t.section_dependencies.items():
                section_dependencies[k].update(v)
        section_flavors = defaultdict(set)
        for target in targets:
            for key, value in target.section_flavors.items():
                section_flavors[key].update(value)
        return Target(name, dependencies, parts, section_dependencies, section_flavors)


@attrs(auto_attribs=True)
class TargetBuilder:
    _data: Dict[str, Any]
    _paths: Paths
    _local_latex_dir: Path = attrib(init=False)

    def __attrs_post_init__(self) -> None:
        self._local_dir = self._paths.current_dir
        self._local_latex_dir = self._local_dir / "latex"

    def build(self) -> Target:
        all_dependencies = Dependencies()
        all_dependencies.unused.update(self._local_latex_dir.glob("**/*.tex"))
        all_items = []
        section_dependencies: DefaultDict[str, Dependencies] = defaultdict(Dependencies)
        section_flavors = defaultdict(set)
        for section_config in self._data["sections"]:
            if not isinstance(section_config, dict):
                section_config = dict(path=section_config)
            section_path = section_config["path"]
            section_flavors[section_path].add(section_config.get("flavor"))
            result = self._parse_section_dir(
                section_path, section_config, 0, section_flavors, section_dependencies
            )
            if result is None:
                result = self._parse_section_file(section_path, section_config, 0)
            if result is not None:
                items, dependencies = result
                all_items.extend(items)
            else:
                dependencies = Dependencies()
                dependencies.missing.add(section_path)
            all_dependencies.update(dependencies)
            section_dependencies[section_path].update(dependencies)
        return Target(
            name=self._data["name"],
            dependencies=all_dependencies,
            parts=[Part(title=self._data["title"], sections=all_items)],
            section_dependencies=section_dependencies,
            section_flavors=section_flavors,
        )

    def _parse_section_dir(
        self,
        section_path_str: str,
        custom_config: Dict[str, Any],
        title_level: int,
        section_flavors: DefaultDict[str, Set[str]],
        section_dependencies: DefaultDict[str, Dependencies],
    ) -> Optional[Tuple[List[ContentOrTitle], Dependencies]]:
        section_path = Path(section_path_str)
        local_section_dir = self._local_latex_dir / section_path
        local_section_config_path = (local_section_dir / section_path).with_suffix(
            ".yml"
        )
        shared_section_dir = self._paths.shared_latex_dir / section_path
        shared_section_config_path = (
            shared_section_dir / section_path.parts[-1]
        ).with_suffix(".yml")
        dependencies = Dependencies()
        if local_section_config_path.exists():
            section_config_path = local_section_config_path
        elif shared_section_config_path.exists():
            section_config_path = shared_section_config_path
        else:
            return None
        section_config = DirSectionConfig.from_yaml_file(section_config_path)
        title = (
            custom_config["title"] if "title" in custom_config else section_config.title
        )
        if "flavor" not in custom_config:
            raise DeckzException(
                f"Incorrect targets {self._paths.targets}. "
                f"Mandatory flavor not specified in {section_path} configuration."
            )
        flavor_name = custom_config["flavor"]
        if flavor_name not in section_config.flavors:
            flavors_string = ", ".join("'%s'" % f for f in section_config.flavors)
            raise DeckzException(
                f"Incorrect targets {self._paths.targets}. "
                f"'{flavor_name}' not amongst available flavors: {flavors_string} "
                f"of {section_config_path}."
            )

        flavor = section_config.flavors[flavor_name]
        items: List[ContentOrTitle] = [Title(title=title, level=title_level)]
        for item in flavor:
            self._process_item(
                item=item,
                items=items,
                dependencies=dependencies,
                default_titles=section_config.default_titles,
                section_config=custom_config,
                local_section_dir=local_section_dir,
                shared_section_dir=shared_section_dir,
                section_path_str=section_path_str,
                title_level=title_level + 1,
                section_flavors=section_flavors,
                section_dependencies=section_dependencies,
            )
        return items, dependencies

    def _process_item(
        self,
        item: Union[str, Dict[str, Optional[str]]],
        items: List[ContentOrTitle],
        dependencies: Dependencies,
        default_titles: Optional[Dict[str, str]],
        section_config: Dict[str, Any],
        local_section_dir: Path,
        shared_section_dir: Path,
        section_path_str: str,
        title_level: int,
        section_flavors: DefaultDict[str, Set[str]],
        section_dependencies: DefaultDict[str, Dependencies],
    ) -> None:
        if isinstance(item, str):
            filename = item
            if default_titles is not None:
                title = default_titles.get(filename)
            else:
                title = None
        else:
            filename, title = next(iter(item.items()))
        if "excludes" in section_config and filename in section_config["excludes"]:
            return
        if filename.startswith("/"):
            local_path = (self._local_latex_dir / filename[1:]).with_suffix(".tex")
            shared_path = (self._paths.shared_latex_dir / filename[1:]).with_suffix(
                ".tex"
            )
        elif filename.startswith("$"):
            assert title is not None
            self._process_nested_section(
                section_path_str=section_path_str,
                nested_section_path_str=filename,
                flavor=title,
                title_level=title_level,
                section_flavors=section_flavors,
                section_dependencies=section_dependencies,
                items=items,
                dependencies=dependencies,
            )
            return
        else:
            local_path = (local_section_dir / filename).with_suffix(".tex")
            shared_path = (shared_section_dir / filename).with_suffix(".tex")
        if title is not None:
            items.append(Title(title=title, level=title_level))
        local_relative_path = local_path.relative_to(self._local_dir)
        shared_relative_path = shared_path.relative_to(self._paths.shared_dir)
        if local_path.exists():
            items.append(str(PurePosixPath(local_relative_path.with_suffix(""))))
            dependencies.used.add(local_path.resolve())
        elif shared_path.exists():
            items.append(str(PurePosixPath(shared_relative_path.with_suffix(""))))
            dependencies.used.add(shared_path.resolve())
        else:
            dependencies.missing.add(filename)

    def _process_nested_section(
        self,
        section_path_str: str,
        nested_section_path_str: str,
        flavor: str,
        title_level: int,
        section_flavors: DefaultDict[str, Set[str]],
        section_dependencies: DefaultDict[str, Dependencies],
        items: List[ContentOrTitle],
        dependencies: Dependencies,
    ) -> None:
        if nested_section_path_str.startswith("$/"):
            nested_section_path = nested_section_path_str[2:]
        else:
            nested_section_path = f"{section_path_str}/{nested_section_path_str[1:]}"
        parsed_nested_dir = self._parse_section_dir(
            nested_section_path,
            dict(flavor=flavor),
            title_level,
            section_flavors,
            section_dependencies,
        )
        assert parsed_nested_dir is not None
        nested_items, nested_dependencies = parsed_nested_dir
        section_flavors[nested_section_path].add(flavor)
        section_dependencies[nested_section_path].update(nested_dependencies)
        items.extend(nested_items)
        dependencies.update(nested_dependencies)

    def _parse_section_file(
        self, section_path: str, config: Dict[str, Any], title_level: int
    ) -> Optional[Tuple[List[ContentOrTitle], Dependencies]]:
        local_section_file = (self._local_latex_dir / section_path).with_suffix(".tex")
        shared_section_file = (self._paths.shared_latex_dir / section_path).with_suffix(
            ".tex"
        )
        if local_section_file.exists():
            section_file = local_section_file
            relative_path = section_file.relative_to(self._local_dir)
        elif shared_section_file.exists():
            section_file = shared_section_file
            relative_path = section_file.relative_to(self._paths.shared_dir)
        else:
            return None
        config_file = section_file.with_suffix(".yml")
        items: List[ContentOrTitle] = []
        if "title" in config:
            title = config["title"]
        elif config_file.exists():
            title = safe_load(config_file.read_text(encoding="utf8"))["title"]
        else:
            title = None
        if title is not None:
            items.append(Title(title=title, level=title_level))
        items.append(str(PurePosixPath(relative_path)))
        dependencies = Dependencies()
        dependencies.used.add(section_file.resolve())
        return items, dependencies


@attrs(auto_attribs=True)
class Targets(Iterable[Target]):
    _targets: List[Target] = attrib()
    _paths: Paths

    @classmethod
    def from_file(
        cls, paths: Paths, whitelist: Optional[List[str]] = None
    ) -> "Targets":
        if not paths.targets.exists():
            raise DeckzException(f"Could not find {paths.targets}.")
        content = safe_load(paths.targets.read_text(encoding="utf8"))
        return cls.from_data(data=content, paths=paths, whitelist=whitelist)

    @classmethod
    def from_data(
        cls,
        data: List[Dict[str, Any]],
        paths: Paths,
        whitelist: Optional[List[str]] = None,
    ) -> "Targets":
        for target_data in data:
            if target_data["name"] == "all":
                raise DeckzException(
                    f"Incorrect targets {paths.targets}: "
                    '"all" is a reserved target name.'
                )
        targets = [TargetBuilder(data=target, paths=paths).build() for target in data]
        if whitelist is not None:
            cls._filter_targets(targets, frozenset(whitelist), paths)
        return Targets(targets, paths)

    @_targets.validator
    def _check_targets(self, attribute: Attribute, value: List[Target]) -> None:
        missing_dependencies = {
            target.name: target.dependencies.missing
            for target in value
            if target.dependencies.missing
        }
        if missing_dependencies:
            raise DeckzException(
                f"Incorrect targets {self._paths.targets}. "
                "Could not find the following dependencies:\n%s"
                % "\n".join(
                    "  - %s:\n%s" % (k, "\n".join(f"    - {p}" for p in v))
                    for k, v in missing_dependencies.items()
                ),
            )

    @staticmethod
    def _filter_targets(
        targets: Iterable[Target], whiteset: FrozenSet[str], paths: Paths
    ) -> List[Target]:
        unmatched = whiteset - frozenset(target.name for target in targets)
        if unmatched:
            raise DeckzException(
                f"Incorrect targets {paths.targets}. "
                "Could not find the following targets:\n%s"
                % "\n".join("  - %s" % name for name in unmatched)
            )
        return [target for target in targets if target.name in whiteset]

    def __iter__(self) -> Iterator[Target]:
        return iter(self._targets)

    def __len__(self) -> int:
        return len(self._targets)



from pathlib import Path

from typer import Option

from deckz.cli import app
from deckz.running import run_all as running_run_all


@app.command()
def check_all(
    handout: bool = Option(False, help="Produce PDFs without animations"),
    presentation: bool = Option(True, help="Produce PDFs with animations"),
    print: bool = Option(False, help="Produce printable PDFs"),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Compile all shared slides (presentation only by default)."""
    running_run_all(
        directory=workdir,
        build_handout=handout,
        build_presentation=presentation,
        build_print=print,
    )



from logging import getLogger
from pathlib import Path
from typing import Optional

from pydantic import BaseModel
from typer import Argument, Option
from yaml import safe_load

from deckz.cli import app
from deckz.github_querying import GitHubAPI
from deckz.paths import GlobalPaths


@app.command()
def issue(
    title: str = Argument(..., help="Title of the issue"),
    body: Optional[str] = Argument(None, help="Body of the issue"),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Create an issue on GitHub."""
    logger = getLogger(__name__)
    config = IssuesConfig.from_global_paths(GlobalPaths.from_defaults(workdir))
    api = GitHubAPI(config.api_key)
    url = api.create_issue(config.owner, config.repo, title, body, config.project)
    logger.info(
        f"Successfully created the issue [link={url}]on GitHub[/link]",
        extra=dict(markup=True),
    )


class IssuesConfig(BaseModel):

    api_key: str
    repo: str
    owner: str
    project: Optional[int] = None

    @classmethod
    def from_global_paths(cls, paths: GlobalPaths) -> "IssuesConfig":
        return cls.parse_obj(safe_load(paths.github_issues.read_text(encoding="utf8")))



from logging import getLogger
from pathlib import Path
from threading import Thread
from time import time
from typing import FrozenSet, List, Optional

from watchdog.events import FileSystemEvent
from watchdog.observers import Observer

from deckz.exceptions import DeckzException
from deckz.paths import GlobalPaths, Paths
from deckz.running import run, run_file, run_standalones

_logger = getLogger(__name__)


class _BaseEventHandler:
    def __init__(self, minimum_delay: int):
        self._minimum_delay = minimum_delay
        self._last_compile = 0.0
        self._worker: Optional[Thread] = None
        self._first_build = True

    def run(self) -> None:
        raise NotImplementedError

    def __call__(self) -> None:
        try:
            self._compiling = True
            if self._first_build:
                self._first_build = False
                _logger.info("Initial build")
            else:
                _logger.info("Detected changes, starting a new build")
            try:
                self.run()
                _logger.info("Build finished")
            except Exception as e:
                _logger.error(str(e), extra=dict(markup=True))
        finally:
            self._compiling = False

    def dispatch(self, event: FileSystemEvent) -> None:
        current_time = time()
        if self._last_compile + self._minimum_delay > current_time:
            return
        elif self._worker is not None and self._worker.is_alive():
            _logger.info("Still on last build, not starting a new build")
            return
        else:
            self._last_compile = current_time
            self._worker = Thread(target=self.__call__)
            self._worker.start()


class _RunnerEventHandler(_BaseEventHandler):
    def __init__(
        self,
        minimum_delay: int,
        paths: Paths,
        build_handout: bool,
        build_presentation: bool,
        build_print: bool,
        target_whitelist: Optional[List[str]],
    ):
        super().__init__(minimum_delay)
        self._paths = paths
        self._build_handout = build_handout
        self._build_presentation = build_presentation
        self._build_print = build_print
        self._target_whitelist = target_whitelist

    def run(self) -> None:
        run(
            paths=self._paths,
            build_handout=self._build_handout,
            build_presentation=self._build_presentation,
            build_print=self._build_print,
            target_whitelist=self._target_whitelist,
        )


class _FileRunnerEventHandler(_BaseEventHandler):
    def __init__(
        self,
        minimum_delay: int,
        latex: str,
        paths: Paths,
        build_handout: bool,
        build_presentation: bool,
        build_print: bool,
    ):
        super().__init__(minimum_delay)
        self._latex = latex
        self._paths = paths
        self._build_handout = build_handout
        self._build_presentation = build_presentation
        self._build_print = build_print

    def run(self) -> None:
        run_file(
            latex=self._latex,
            paths=self._paths,
            build_handout=self._build_handout,
            build_presentation=self._build_presentation,
            build_print=self._build_print,
        )


class _StandalonesRunnerEventHandler(_BaseEventHandler):
    def __init__(self, minimum_delay: int, current_dir: Path):
        super().__init__(minimum_delay)
        self._current_dir = current_dir

    def run(self) -> None:
        run_standalones(self._current_dir)


def _watch(
    event_handler: _BaseEventHandler, watch: FrozenSet[Path], avoid: FrozenSet[Path]
) -> None:
    observer = Observer()
    dirs_to_avoid = avoid | {
        r_to_avoid
        for dir_to_avoid in avoid
        for p in dir_to_avoid.glob("**/*")
        if (r_to_avoid := p.resolve()).is_dir()
    }
    dirs_to_watch = watch | {
        r_to_watch
        for dir_to_watch in watch
        for p in dir_to_watch.glob("**/*")
        if (r_to_watch := p.resolve()).is_dir() and r_to_watch not in dirs_to_avoid
    }
    for dir_to_watch in dirs_to_watch:
        observer.schedule(event_handler, str(dir_to_watch.resolve()), recursive=False)
    event_handler()
    observer.start()
    try:
        observer.join()
    except KeyboardInterrupt:
        observer.stop()
        observer.join()
        _logger.info("Stopped watching")
    else:
        observer.join()
        raise DeckzException("Stopped watching abnormally")


def watch(
    minimum_delay: int,
    paths: Paths,
    build_handout: bool,
    build_presentation: bool,
    build_print: bool,
    target_whitelist: Optional[List[str]],
) -> None:
    _logger.info("Watching the shared, current and user directories")
    _watch(
        _RunnerEventHandler(
            minimum_delay,
            paths,
            build_handout=build_handout,
            build_presentation=build_presentation,
            build_print=build_print,
            target_whitelist=target_whitelist,
        ),
        watch=frozenset([paths.shared_dir, paths.current_dir, paths.user_config_dir]),
        avoid=frozenset([paths.shared_tikz_pdf_dir, paths.pdf_dir, paths.build_dir]),
    )


def watch_file(
    minimum_delay: int,
    latex: str,
    paths: Paths,
    build_handout: bool,
    build_presentation: bool,
    build_print: bool,
) -> None:
    _logger.info("Watching the shared and user directories")
    _watch(
        _FileRunnerEventHandler(
            minimum_delay,
            latex,
            paths,
            build_handout=build_handout,
            build_presentation=build_presentation,
            build_print=build_print,
        ),
        watch=frozenset([paths.shared_dir, paths.user_config_dir]),
        avoid=frozenset([paths.shared_tikz_pdf_dir, paths.pdf_dir, paths.build_dir]),
    )


def watch_standalones(
    minimum_delay: int, current_dir: Path, paths: GlobalPaths
) -> None:
    _logger.info("Watching the shared tikz directory")
    _watch(
        _StandalonesRunnerEventHandler(minimum_delay, current_dir),
        watch=frozenset([paths.shared_tikz_dir]),
        avoid=frozenset([paths.shared_tikz_pdf_dir]),
    )



from logging import getLogger
from pathlib import Path
from tempfile import TemporaryDirectory

from typer import Argument, Option

from deckz import app_name
from deckz.cli import app
from deckz.paths import GlobalPaths, Paths
from deckz.watching import watch_file as watching_watch_file

_logger = getLogger(__name__)


@app.command()
def watch_file(
    latex: str = Argument(..., help="File to watch, relative to share/latex"),
    handout: bool = Option(False, help="Produce PDFs without animations"),
    presentation: bool = Option(True, help="Produce PDFs with animations"),
    print: bool = Option(False, help="Produce a printable PDF"),
    minimum_delay: int = Option(5, help="Minimum number of seconds before recompiling"),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Compile a specific file on change."""
    _logger.info(f"Watching {latex}")
    global_paths = GlobalPaths.from_defaults(workdir)
    with TemporaryDirectory(prefix=f"{app_name}-") as build_dir, TemporaryDirectory(
        prefix=f"{app_name}-"
    ) as pdf_dir:
        _logger.info(
            f"Output directory located at [link=file://{pdf_dir}]{pdf_dir}[/link]",
            extra=dict(markup=True),
        )
        watching_watch_file(
            minimum_delay=minimum_delay,
            latex=latex,
            paths=Paths.from_defaults(
                workdir,
                check_depth=False,
                build_dir=Path(build_dir),
                pdf_dir=Path(pdf_dir),
                company_config=global_paths.template_company_config,
                deck_config=global_paths.template_deck_config,
            ),
            build_handout=handout,
            build_presentation=presentation,
            build_print=print,
        )



from logging import getLogger
from pathlib import Path
from typing import Container, Dict, Type, TypeVar, Union

from appdirs import user_config_dir as appdirs_user_config_dir
from attr import attrib, attrs

from deckz import app_name
from deckz.exceptions import DeckzException
from deckz.utils import get_git_dir

_logger = getLogger(__name__)


def _path_converter(path: Union[str, Path]) -> Path:
    return Path(path).resolve()


_GlobalPathsType = TypeVar("_GlobalPathsType", bound="GlobalPaths")


@attrs(auto_attribs=True)
class GlobalPaths:
    current_dir: Path = attrib(converter=_path_converter)
    git_dir: Path = attrib(converter=_path_converter)
    settings: Path = attrib(converter=_path_converter)
    shared_dir: Path = attrib(converter=_path_converter)
    shared_img_dir: Path = attrib(converter=_path_converter)
    shared_code_dir: Path = attrib(converter=_path_converter)
    shared_latex_dir: Path = attrib(converter=_path_converter)
    shared_tikz_dir: Path = attrib(converter=_path_converter)
    shared_tikz_pdf_dir: Path = attrib(converter=_path_converter)
    templates_dir: Path = attrib(converter=_path_converter)
    yml_templates_dir: Path = attrib(converter=_path_converter)
    template_global_config: Path = attrib(converter=_path_converter)
    template_user_config: Path = attrib(converter=_path_converter)
    template_company_config: Path = attrib(converter=_path_converter)
    template_deck_config: Path = attrib(converter=_path_converter)
    jinja2_dir: Path = attrib(converter=_path_converter)
    jinja2_main_template: Path = attrib(converter=_path_converter)
    jinja2_print_template: Path = attrib(converter=_path_converter)
    user_config_dir: Path = attrib(converter=_path_converter)
    global_config: Path = attrib(converter=_path_converter)
    github_issues: Path = attrib(converter=_path_converter)
    mails: Path = attrib(converter=_path_converter)
    gdrive_secrets: Path = attrib(converter=_path_converter)
    gdrive_credentials: Path = attrib(converter=_path_converter)
    user_config: Path = attrib(converter=_path_converter)

    def __attrs_post_init__(self) -> None:
        self.user_config_dir.mkdir(parents=True, exist_ok=True)

    @classmethod
    def _defaults_global_paths(cls, current_dir: Path) -> Dict[str, Path]:
        current_dir = current_dir.resolve()
        git_dir = get_git_dir(current_dir)
        shared_dir = git_dir / "shared"
        templates_dir = git_dir / "templates"
        yml_templates_dir = templates_dir / "yml"
        jinja2_dir = templates_dir / "jinja2"
        user_config_dir = Path(appdirs_user_config_dir(app_name))
        return dict(
            current_dir=current_dir,
            git_dir=git_dir,
            shared_dir=shared_dir,
            templates_dir=templates_dir,
            yml_templates_dir=yml_templates_dir,
            jinja2_dir=jinja2_dir,
            user_config_dir=user_config_dir,
            shared_tikz_dir=shared_dir / "tikz-src",
            shared_tikz_pdf_dir=shared_dir / "tikz",
            settings=git_dir / "settings.yml",
            shared_img_dir=shared_dir / "img",
            shared_code_dir=shared_dir / "code",
            shared_latex_dir=shared_dir / "latex",
            template_global_config=yml_templates_dir / "global-config.yml",
            template_user_config=yml_templates_dir / "user-config.yml",
            template_company_config=yml_templates_dir / "company-config.yml",
            template_deck_config=yml_templates_dir / "deck-config.yml",
            jinja2_main_template=jinja2_dir / "main.tex",
            jinja2_print_template=jinja2_dir / "print.tex",
            global_config=git_dir / "global-config.yml",
            github_issues=user_config_dir / "github-issues.yml",
            mails=user_config_dir / "mails.yml",
            gdrive_secrets=user_config_dir / "gdrive-secrets.json",
            gdrive_credentials=user_config_dir / "gdrive-credentials.pickle",
            user_config=user_config_dir / "user-config.yml",
        )

    @classmethod
    def from_defaults(
        cls: Type[_GlobalPathsType],
        current_dir: Path,
        check_depth: bool = True,
        **kwargs: Path,
    ) -> _GlobalPathsType:
        return cls(**{**cls._defaults_global_paths(current_dir), **kwargs})


_PathsType = TypeVar("_PathsType", bound="Paths")


@attrs(auto_attribs=True)
class Paths(GlobalPaths):
    build_dir: Path = attrib(converter=_path_converter)
    pdf_dir: Path = attrib(converter=_path_converter)
    company_config: Path = attrib(converter=_path_converter)
    deck_config: Path = attrib(converter=_path_converter)
    session_config: Path = attrib(converter=_path_converter)
    targets: Path = attrib(converter=_path_converter)

    @classmethod
    def _defaults_paths(
        cls, current_dir: Path, check_depth: bool, skip: Container[str]
    ) -> Dict[str, Path]:
        defaults = super()._defaults_global_paths(current_dir)
        if check_depth and not defaults["current_dir"].relative_to(
            defaults["git_dir"]
        ).match("*/*"):
            raise DeckzException(
                f"Not deep enough from root {defaults['git_dir']}. "
                "Please follow the directory hierarchy root > company > deck and "
                "invoke this tool from the deck directory."
            )
        additional_defaults_items = dict(
            build_dir=lambda: defaults["current_dir"] / ".build",
            pdf_dir=lambda: defaults["current_dir"] / "pdf",
            company_config=lambda: (
                defaults["git_dir"]
                / defaults["current_dir"].relative_to(defaults["git_dir"]).parts[0]
                / "company-config.yml"
            ),
            deck_config=lambda: defaults["current_dir"] / "deck-config.yml",
            session_config=lambda: defaults["current_dir"] / "session-config.yml",
            targets=lambda: defaults["current_dir"] / "targets.yml",
        )
        additional_defaults = {
            key: value_function()
            for key, value_function in additional_defaults_items.items()
            if key not in skip
        }
        return {**defaults, **additional_defaults}

    @classmethod
    def from_defaults(
        cls: Type[_PathsType],
        current_dir: Path,
        check_depth: bool = True,
        **kwargs: Path,
    ) -> _PathsType:
        return cls(
            **{
                **cls._defaults_paths(
                    current_dir,
                    check_depth=check_depth,
                    skip=kwargs,
                ),
                **kwargs,
            }
        )



from logging import getLogger
from pathlib import Path

from typer import Option

from deckz.cli import app
from deckz.paths import Paths
from deckz.uploading import Uploader

_logger = getLogger(__name__)


@app.command()
def upload(
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    )
) -> None:
    """Upload pdfs to Google Drive."""
    paths = Paths.from_defaults(workdir)
    Uploader(paths)



from importlib import import_module
from importlib import invalidate_caches as importlib_invalidate_caches
from logging import INFO, basicConfig
from pkgutil import walk_packages

from rich.logging import RichHandler
from typer import Typer

app = Typer(
    help="Tool to handle a large number of beamer decks, "
    "used by several persons, with shared slides amongst the decks.",
    chain=True,
)


def main() -> None:
    basicConfig(
        level=INFO,
        format="%(message)s",
        datefmt="%H:%M:%S",
        handlers=[RichHandler(rich_tracebacks=True)],
    )
    _import_module_and_submodules(__name__)
    app()


def _import_module_and_submodules(package_name: str) -> None:
    """
    From https://github.com/allenai/allennlp/blob/master/allennlp/common/util.py
    """
    importlib_invalidate_caches()

    module = import_module(package_name)
    path = getattr(module, "__path__", [])
    path_string = "" if not path else path[0]

    for module_finder, name, _ in walk_packages(path):
        if (
            path_string
            and hasattr(module_finder, "path")
            and module_finder.path != path_string  # type: ignore
        ):
            continue
        subpackage = f"{package_name}.{name}"
        _import_module_and_submodules(subpackage)



from logging import getLogger
from pathlib import Path
from shutil import rmtree

from typer import Option

from deckz.cli import app
from deckz.paths import Paths


@app.command()
def clean(
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    )
) -> None:
    """Wipe the build directory."""
    logger = getLogger(__name__)
    paths = Paths.from_defaults(workdir)
    if not paths.build_dir.exists():
        logger.info(f"Nothing to do: {paths.build_dir} doesn't exist")
    else:
        logger.info(f"Deleting {paths.build_dir}")
        rmtree(paths.build_dir)



from typing import Dict, List

from pydantic import BaseModel
from yaml import safe_load

from deckz.exceptions import DeckzException
from deckz.paths import GlobalPaths


class LocalizedValues(BaseModel):
    fr: Dict[str, str] = {}
    en: Dict[str, str] = {}
    all: Dict[str, str] = {}

    def get_default(self, value: str, lang: str) -> str:
        if lang == "fr" and value in self.fr:
            return self.fr[value]
        if lang == "en" and value in self.en:
            return self.en[value]
        return self.all[value] if value in self.all else value


class DefaultImageValues(BaseModel):
    license: LocalizedValues = LocalizedValues()
    author: LocalizedValues = LocalizedValues()
    title: LocalizedValues = LocalizedValues()


class Settings(BaseModel):

    build_command: List[str]
    default_img_values: DefaultImageValues = DefaultImageValues()

    @classmethod
    def from_global_paths(cls, global_paths: GlobalPaths) -> "Settings":
        if not global_paths.settings.is_file():
            raise DeckzException(
                f"Could not find settings file at {global_paths.settings}."
            )
        return cls.parse_obj(
            safe_load(global_paths.settings.read_text(encoding="utf8"))
        )



from logging import getLogger
from pathlib import Path
from shutil import rmtree

from typer import Option

from deckz.cli import app
from deckz.paths import GlobalPaths, Paths


@app.command()
def clean_all(
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    )
) -> None:
    """Wipe all build directories."""
    logger = getLogger(__name__)
    paths = GlobalPaths.from_defaults(workdir)
    for deck_path in (p.parent for p in paths.git_dir.glob("**/targets.yml")):
        deck_paths = Paths.from_defaults(deck_path)
        if not deck_paths.build_dir.exists():
            logger.info(f"Nothing to do: {deck_paths.build_dir} doesn't exist")
        else:
            logger.info(f"Deleting {deck_paths.build_dir}")
            rmtree(deck_paths.build_dir)



from logging import getLogger
from pathlib import Path
from random import choice
from typing import Dict

from pydantic import BaseModel, EmailStr
from rich.console import Console
from rich.markdown import Markdown
from rich.prompt import Prompt
from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import Mail
from typer import Argument, Option
from yaml import safe_load

from deckz.cli import app
from deckz.paths import GlobalPaths


@app.command()
def random(
    reason: str = Argument(..., help='Reason for the deckz random ("Pay the bill")'),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Roll the dice and email the result."""
    logger = getLogger(__name__)
    config = MailsConfig.from_global_paths(GlobalPaths.from_defaults(workdir))
    console = Console()
    names = list(config.to)
    names_list_str = "\n".join(
        f"{i}. {name} <{config.to[name]}>" for i, name in enumerate(names, start=1)
    )
    console.print(Markdown(f"Here are the possible participants:\n\n{names_list_str}"))
    print()
    ok = False
    default = ", ".join(map(str, range(1, len(names) + 1)))
    while not ok:
        answer = Prompt().ask(
            "Please enter a comma separated list of numbers to select participants "
            f"(default: {default})",
            default=default,
        )
        numbers_str = answer.split(",")
        try:
            numbers = [int(number_str.strip()) for number_str in numbers_str]
            if not all(0 < n <= len(names) for n in numbers):
                raise ValueError(
                    "All numbers should be between 1 and the number of participants"
                )
            ok = True
        except Exception:
            console.print("Could not parse your selection, please try again.")
    selected_names = [names[i - 1] for i in numbers]
    name = choice(selected_names)
    draw_info = f"{name} was drawn randomly from {', '.join(selected_names)}"
    logger.info(draw_info)
    sendgrid_email = Mail(
        from_email=config.mail,
        to_emails=list(config.to.values()),
        subject=f"[deckz random] {reason}: {name} got picked",
        plain_text_content=draw_info,
    )
    client = SendGridAPIClient(config.api_key)
    client.send(sendgrid_email)


class MailsConfig(BaseModel):

    api_key: str
    mail: str
    to: Dict[str, EmailStr]

    @classmethod
    def from_global_paths(cls, paths: GlobalPaths) -> "MailsConfig":
        return cls.parse_obj(safe_load(paths.mails.read_text(encoding="utf8")))



from contextlib import redirect_stdout
from functools import partial
from itertools import chain
from logging import getLogger
from multiprocessing import Pool
from pathlib import Path
from shutil import copyfile
from tempfile import TemporaryDirectory

from deckz.compiling import CompilePaths
from deckz.compiling import compile as compiling_compile
from deckz.exceptions import DeckzException
from deckz.paths import GlobalPaths
from deckz.settings import Settings
from deckz.utils import copy_file_if_newer


class StandalonesBuilder:
    def __init__(self, settings: Settings, paths: GlobalPaths):
        self._paths = paths
        self._settings = settings
        self._logger = getLogger(__name__)

    def build(self) -> None:
        self._logger.info("Processing standalones")
        with TemporaryDirectory() as build_dir:
            build_path = Path(build_dir)
            items = [
                (input_path, paths)
                for input_path in chain(
                    self._paths.shared_tikz_dir.glob("**/*.py"),
                    self._paths.shared_tikz_dir.glob("**/*.tex"),
                )
                if self._needs_compile(
                    input_path,
                    paths := self._compute_compile_paths(input_path, build_path),
                )
            ]

            for item in items:
                self._prepare(*item)

            with Pool() as pool:
                results = pool.map(
                    partial(compiling_compile, settings=self._settings),
                    (item_path.latex for _, item_path in items),
                )

            for (_, paths), result in zip(items, results):
                if result.ok:
                    paths.output_pdf.parent.mkdir(parents=True, exist_ok=True)
                    copyfile(paths.build_pdf, paths.output_pdf)
                    paths.output_log.unlink(missing_ok=True)
                elif paths.build_log.exists():
                    paths.output_pdf.parent.mkdir(parents=True, exist_ok=True)
                    copyfile(paths.build_log, paths.output_log)

        failed = []
        for (input_path, paths), result in zip(items, results):
            if not result.ok:
                failed.append((input_path, paths.output_log))
                self._logger.warning("Standalone compilation of %s errored", input_path)
                self._logger.warning("Captured stderr\n%s", result.stderr)
        if failed:
            formatted_fails = "\n".join(
                "- %s (%s)"
                % (
                    f.relative_to(self._paths.shared_dir),
                    "no log" if not l.exists() else "[link=file://%s]log[/link]" % l,
                )
                for f, l in failed
            )
            raise DeckzException(
                f"Standalone compilation errored for {len(failed)} files:\n"
                f"{formatted_fails}\n"
                "Please also check the errors above."
            )

    def _needs_compile(self, input_file: Path, compile_paths: CompilePaths) -> bool:
        return (
            not compile_paths.output_pdf.exists()
            or compile_paths.output_pdf.stat().st_mtime < input_file.stat().st_mtime
        )

    def _generate_latex(self, python_file: Path, output_file: Path) -> None:
        compiled = compile(
            source=python_file.read_text(encoding="utf8"),
            filename=python_file.name,
            mode="exec",
        )
        output_file.parent.mkdir(parents=True, exist_ok=True)
        with output_file.open("w", encoding="utf8") as fh:
            with redirect_stdout(fh):
                exec(compiled)

    def _compute_compile_paths(self, input_file: Path, build_dir: Path) -> CompilePaths:
        latex = (
            build_dir / input_file.relative_to(self._paths.shared_tikz_dir)
        ).with_suffix(".tex")
        build_pdf = latex.with_suffix(".pdf")
        output_pdf = (
            self._paths.shared_tikz_pdf_dir
            / input_file.relative_to(self._paths.shared_tikz_dir)
        ).with_suffix(".pdf")
        build_log = latex.with_suffix(".log")
        output_log = output_pdf.with_suffix(".log")
        return CompilePaths(
            latex=latex,
            build_pdf=build_pdf,
            output_pdf=output_pdf,
            build_log=build_log,
            output_log=output_log,
        )

    def _prepare(self, input_file: Path, compile_paths: CompilePaths) -> None:
        build_dir = compile_paths.latex.parent
        build_dir.mkdir(parents=True, exist_ok=True)
        build_img_dir = build_dir / self._paths.shared_img_dir.name
        if not build_img_dir.exists():
            build_img_dir.symlink_to(self._paths.shared_img_dir)

        if input_file.suffix == ".py":
            self._generate_latex(input_file, compile_paths.latex)
        elif input_file.suffix == ".tex":
            copy_file_if_newer(input_file, compile_paths.latex)
        else:
            raise ValueError(
                f"Unsupported standalone file extension {input_file.suffix}"
            )



from logging import getLogger
from pathlib import Path

from typer import Option

from deckz.cli import app
from deckz.config import get_config
from deckz.paths import Paths


@app.command()
def print_config(
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    )
) -> None:
    """Print the resolved configuration."""
    logger = getLogger(__name__)
    paths = Paths.from_defaults(workdir)
    config = get_config(paths)
    logger.info(
        "Resolved config as:\n%s",
        "\n".join((f"  - {k}: {v}") for k, v in config.items()),
    )



from logging import getLogger
from pathlib import Path
from pickle import dump as pickle_dump
from pickle import load as pickle_load
from typing import Any, Dict, List, Optional, Tuple

from google.auth.transport.requests import Request
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload
from rich.progress import (
    BarColumn,
    DownloadColumn,
    Progress,
    TaskID,
    TextColumn,
    TimeRemainingColumn,
    TransferSpeedColumn,
)

from deckz import app_name
from deckz.exceptions import DeckzException
from deckz.paths import Paths


class Uploader:
    def __init__(self, paths: Paths):
        self._logger = getLogger(__name__)
        self._paths = paths
        self._service = self._build_service()
        folder_id, folder_link = self._check_folders()
        backup_id = self._create_backup(folder_id)
        self._upload(folder_id)
        if backup_id:
            self._logger.info("Deleting backup of old files")
            self._service.files().delete(fileId=backup_id).execute()
        print(f"Online folder: {folder_link}")

    @staticmethod
    def _build_progress() -> Progress:
        return Progress(
            TextColumn("[bold blue]{task.fields[filename]}", justify="right"),
            BarColumn(bar_width=None),
            "[progress.percentage]{task.percentage:>3.1f}%",
            "•",
            DownloadColumn(),
            "•",
            TransferSpeedColumn(),
            "•",
            TimeRemainingColumn(),
        )

    @staticmethod
    def _build_task(progress: Progress, filename: str, size: int) -> TaskID:
        return progress.add_task("upload", filename=filename, total=size)

    def _build_service(self) -> Any:
        if self._paths.gdrive_credentials.is_file():
            with self._paths.gdrive_credentials.open("rb") as fh:
                creds = pickle_load(fh)
        else:
            creds = None

        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    self._paths.gdrive_secrets,
                    ["https://www.googleapis.com/auth/drive.file"],
                )
                creds = flow.run_local_server(port=0)
            with self._paths.gdrive_credentials.open("wb") as fh:
                pickle_dump(creds, fh)

        return build("drive", "v3", credentials=creds, cache_discovery=False)

    def _check_folders(self) -> Tuple[str, str]:
        self._logger.info("Checking/creating folder hierarchy")
        folders = [app_name]
        folders.extend(self._paths.current_dir.relative_to(self._paths.git_dir).parts)
        parent = "root"
        for folder in folders:
            folder_info = self._get(folder=True, parents=[parent], name=folder)
            if folder_info is None:
                folder_id, folder_link = self._create_folder(parent, folder)
                self._logger.debug(f"“{folder}” folder was created")
            else:
                folder_id, folder_link = (
                    folder_info.get("id"),
                    folder_info.get("webViewLink"),
                )
                self._logger.debug(f"“{folder}” folder was present")
            parent = folder_id
        self._logger.debug(f"Setting permissions for {folder}")
        self._service.permissions().create(
            fileId=folder_id, body={"type": "anyone", "role": "reader"}
        ).execute()
        return folder_id, folder_link

    def _create_backup(self, folder_id: str) -> Optional[str]:
        file_ids = self._list(folder=False, parents=[folder_id], name=None)
        backup_id = None
        if file_ids:
            self._logger.info("Creating backup of current files")
            old_backup_info = self._get(folder=True, parents=[folder_id], name="backup")
            if old_backup_info is not None:
                old_backup_id = old_backup_info.get("id")
                self._service.files().update(
                    fileId=old_backup_id, body=dict(name="backup-old")
                ).execute()
            backup_id, _ = self._create_folder(parent=folder_id, name="backup")
            for file_id in file_ids:
                self._service.files().update(
                    fileId=file_id, addParents=backup_id, removeParents=folder_id
                ).execute()
            if old_backup_info is not None:
                old_backup_id = old_backup_info.get("id")
                self._service.files().delete(fileId=old_backup_id).execute()
        return backup_id

    def _upload(self, folder_id: str) -> Dict[Path, str]:
        self._logger.info("Uploading pdfs")
        pdfs = sorted((self._paths.pdf_dir).glob("*.pdf"), key=lambda p: p.name)
        links: Dict[Path, str] = {}
        progress = self._build_progress()
        with progress:
            for pdf in pdfs:
                pdf_size = pdf.stat().st_size
                file_metadata = dict(name=pdf.name, parents=[folder_id])
                media = MediaFileUpload(
                    str(pdf),
                    chunksize=256 * 1024,
                    mimetype="application/pdf",
                    resumable=True,
                )
                request = self._service.files().create(
                    body=file_metadata, media_body=media, fields="id,webViewLink"
                )
                response = None
                task = self._build_task(progress, pdf.name, pdf_size)
                previous_progress = 0
                while response is None:
                    status, response = request.next_chunk()
                    if status and previous_progress != status.progress():
                        progress.update(
                            task,
                            advance=int(
                                (status.progress() - previous_progress) * pdf_size
                            ),
                        )
                        previous_progress = status.progress()
                progress.update(task, completed=pdf_size)
                links[pdf] = response.get("webViewLink")
        return links

    def _create_folder(self, parent: str, name: str) -> Tuple[str, str]:
        file_metadata = dict(
            name=name, mimeType="application/vnd.google-apps.folder", parents=[parent]
        )
        file = (
            self._service.files()
            .create(body=file_metadata, fields="id,webViewLink")
            .execute()
        )
        return file.get("id"), file.get("webViewLink")

    def _list(
        self,
        folder: Optional[bool],
        parents: List[str],
        name: Optional[str],
    ) -> List[str]:
        return [item.get("id") for item in self._query(folder, parents, name)]

    def _get(
        self, folder: Optional[bool], parents: List[str], name: Optional[str]
    ) -> Optional[Any]:
        results = self._query(folder, parents, name)
        if len(results) > 1:
            raise DeckzException("Found several files while trying to retrieve one.")
        return results[0] if results else None

    def _query(
        self, folder: Optional[bool], parents: List[str], name: Optional[str]
    ) -> Any:
        page_token = None
        results = []
        query_conditions = ["trashed = false"]
        if folder is not None:
            query_conditions.append(
                f"mimeType {'=' if folder else '!='} "
                "'application/vnd.google-apps.folder'"
            )
        for parent in parents:
            query_conditions.append(f"'{parent}' in parents")
        if name is not None:
            query_conditions.append(f"name = '{name}'")
        query = " and ".join(query_conditions)
        while True:
            response = (
                self._service.files()
                .list(
                    q=query,
                    spaces="drive",
                    fields="nextPageToken, files(id,webViewLink)",
                    pageToken=page_token,
                )
                .execute()
            )
            results.extend(response.get("files", []))
            page_token = response.get("nextPageToken", None)
            if page_token is None:
                break
        return results



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['deckz', 'deckz.cli']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=2.11.3,<3.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'appdirs>=1.4.4,<2.0.0',
 'attrs>=21.2.0,<22.0.0',
 'email-validator>=1.1.3,<2.0.0',
 'google-api-python-client>=2.3.0,<3.0.0',
 'google-auth-oauthlib>=0.4.4,<0.5.0',
 'pydantic>=1.8.1,<2.0.0',
 'pygit2>=1.5.0,<2.0.0',
 'rich>=10.1.0,<11.0.0',
 'sendgrid>=6.7.0,<7.0.0',
 'typer[all]>=0.3.2,<0.4.0',
 'watchdog>=2.1.1,<3.0.0']

entry_points = \
{'console_scripts': ['deckz = deckz.cli:main']}

setup_kwargs = {
    'name': 'deckz',
    'version': '16.0.0',
    'description': 'Tool to handle multiple beamer decks.',
    'long_description': "# `deckz`\n\n[![CI Status](https://img.shields.io/github/workflow/status/nzmognzmp/deckz/CI?label=CI&style=for-the-badge)](https://github.com/nzmognzmp/deckz/actions?query=workflow%3ACI)\n[![CD Status](https://img.shields.io/github/workflow/status/nzmognzmp/deckz/CD?label=CD&style=for-the-badge)](https://github.com/nzmognzmp/deckz/actions?query=workflow%3ACD)\n[![Test Coverage](https://img.shields.io/codecov/c/github/nzmognzmp/deckz?style=for-the-badge)](https://codecov.io/gh/nzmognzmp/deckz)\n[![PyPI Project](https://img.shields.io/pypi/v/deckz?style=for-the-badge)](https://pypi.org/project/deckz/)\n\nTool to handle a large number of beamer decks, used by several persons, with shared slides amongst the decks. It is currently not meant to be usable directly by people finding about the package on GitHub. Please open an issue if you want more details or want to discuss this solution.\n\n## Installation\n\nWith `pip`:\n\n```shell\npip install deckz\n```\n\n### Shell completion installation\n\nSee the `--show-completion` or `--install-completion` options of the `deckz` CLI.\n\n## Directory Structure\n\n`deckz` works with big assumptions on the directory structure of your presentation repository. Among those assumptions:\n\n- your directory should be a git repository\n- it should contain a `shared` folder for everything that will be shared by all decks during compilation (images, code snippets, etc)\n- it should contain jinja2 LaTeX templates in the `templates/jinja2` directory, with a specific name (listed below)\n- it should contain YAML templates in the `templates/yml` directory, with specific names (listed below)\n- your deck folders should be contained in an organization/company folder. This is meant to avoid repeating the company details all over the place\n- several configuration should be present to customize the decks efficiently (more on that later)\n\n```text\nroot (git repository)\n├── global-config.yml\n├── templates\n│\xa0\xa0 ├── jinja2\n│   │   ├── main.tex\n│   │   └── print.tex\n│\xa0\xa0 └── yml\n│       ├── company-config.yml\n│       ├── deck-config.yml\n│       ├── global-config.yml\n│       ├── targets.yml\n│       └── user-config.yml\n├── shared\n│\xa0\xa0 ├── img\n│\xa0\xa0 │\xa0\xa0 ├── image1.png\n│\xa0\xa0 │\xa0\xa0 └── image2.jpg\n│\xa0\xa0 ├── code\n│\xa0\xa0 │\xa0\xa0 ├── snippet1.py\n│\xa0\xa0 │\xa0\xa0 └── snippet2.js\n│\xa0\xa0 └── latex\n│\xa0\xa0  \xa0\xa0 ├── module1.tex\n│\xa0\xa0  \xa0\xa0 └── module2.tex\n├── company1\n│\xa0\xa0 ├── company-config.yml\n│\xa0\xa0 └── deck1\n│\xa0\xa0  \xa0\xa0 ├── session-config.yml\n│\xa0\xa0  \xa0\xa0 ├── deck-config.yml\n│\xa0\xa0  \xa0\xa0 └── targets.yml\n└── company2\n \xa0\xa0 ├── company-config.yml\n \xa0\xa0 └── deck2\n        ├── target1\n        │   └── custom-module.tex\n \xa0\xa0  \xa0\xa0 ├── deck-config.yml\n \xa0\xa0  \xa0\xa0 └── targets.yml\n```\n\n## Configuration\n\n`deckz` uses small configuration files in several places to avoid repetition.\n\n### Configuration merging\n\nThe configuration are merged in this order (a value from a configuration on the bottom overrides a value from a configuration on the top):\n\n- `global-config.yml`\n- `user-config.yml`\n- `company-config.yml`\n- `deck-config.yml`\n- `session-config.yml`\n\n### Using the configuration values in LaTeX files\n\nThe values obtained from the merged configurations can be used in LaTeX after a conversion from snake case to camel case: if the configuration contains the key `trainer_email`, it will be defined as the `\\TrainerEmail` command in LaTeX.\n\n### Details about specific configurations\n\n#### Global configuration\n\nThe global configuration contains the default values that don't fit at a more specific level.\n\nExample:\n\n```yml\npresentation_size: 10pt\n```\n\n#### User configuration\n\nThe user configuration contains the values that change when the speaker changes. It is located in the XDG compliant config location. It is `$HOME/.config/deckz/user-config.yml` on GNU/Linux for example.\n\nExample:\n\n```yml\ntrainer_activity: Data Scientist\ntrainer_email: john@doe.me\ntrainer_name: John Doe\ntrainer_specialization: NLP, NLU\ntrainer_training: MSc at UCL\n```\n\n#### Company configuration\n\nThe company configuration contains everything required to brand the presentations according to the represented company.\n\nExample:\n\n```yml\ncompany_logo: logo_company\ncompany_logo_height: 1cm\ncompany_name: Company\ncompany_website: https://www.company.com\n```\n\n#### Deck configuration\n\nThe deck configuration contains the title and acronym of the talk.\n\nExample:\n\n```yml\ndeck_acronym: COV19\ndeck_title: Machine Learning and COVID-19\n```\n\n#### Session configuration\n\nThe session configuration is optional and contains everything that will change from one session of a specific talk to another one.\n\nExample:\n\n```yml\nsession_end: 30/04/2020\nsession_start: 27/04/2020\n```\n\n## Usage\n\nSee the `--help` flag of the `deckz` command line tool.\n",
    'author': 'm09',
    'author_email': '142691+m09@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mlambda/deckz',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path
from re import compile as re_compile

from typer import Argument, Option

from deckz.cli import app
from deckz.paths import GlobalPaths


@app.command()
def img_deps(
    image: str = Argument(
        ..., help="Image to track, like img/turing or tikz/variables"
    ),
    workdir: Path = Option(
        Path("."), help="Path to move into before running the command"
    ),
) -> None:
    """Find which latex files use an image."""
    paths = GlobalPaths.from_defaults(workdir)
    pattern = re_compile(fr'(\\V{{\[?"{image}".*\]? \| image}})')
    shared_latex_dir = paths.shared_latex_dir
    for f in shared_latex_dir.rglob("*.tex"):
        if pattern.search(f.read_text(encoding="utf8")):
            print(f.relative_to(shared_latex_dir))



from pathlib import Path
from subprocess import run
from typing import Optional

from attr import attrib, attrs

from deckz.settings import Settings


@attrs(auto_attribs=True, frozen=True)
class CompilePaths:
    latex: Path
    build_pdf: Path
    output_pdf: Path
    build_log: Path
    output_log: Path


@attrs(auto_attribs=True, frozen=True)
class CompileResult:
    ok: bool
    stdout: Optional[str] = attrib(default="")
    stderr: Optional[str] = attrib(default="")


def compile(latex_path: Path, settings: Settings) -> CompileResult:
    completed_process = run(
        settings.build_command + [latex_path.name],
        cwd=latex_path.parent,
        capture_output=True,
        encoding="utf8",
    )
    return CompileResult(
        completed_process.returncode == 0,
        completed_process.stdout,
        completed_process.stderr,
    )
